import java.util.concurrent.locks.*;

public class BoundedQueue<E> {
    private Object[] elements;
    private int front, rear, numItems;
    private Lock queueLock = new ReentrantLock();
    private Condition spaceAvailableCondition = queueLock.newCondition();
    private Condition valueAvailableCondition = queueLock.newCondition();
    
    public BoundedQueue(int capacity)    {
        elements = new Object[capacity];
        front = numItems = 0; rear = -1;
    }
    public E remove() throws InterruptedException {
        queueLock.lock();
        try {
            while (numItems == 0)
                valueAvailableCondition.await();
            E r = (E) elements[front];
            front = (front+1)%elements.length;
            numItems--;
            spaceAvailableCondition.signalAll();
            return r;
        } finally {
            queueLock.unlock();
        }
    }
    public void add(E newValue) throws InterruptedException {
        queueLock.lock();
        try {
            while (numItems == elements.length)
                spaceAvailableCondition.await();
            rear = (rear+1)%elements.length;
            elements[rear] = newValue;
            numItems++;
            valueAvailableCondition.signalAll();
        } finally {
            queueLock.unlock();
        }
    }
}
